/* REXX -- SendMail.cmd -- send e-mail to a POP server */

signal on Error
signal on FAILURE name Error
signal on Halt
signal on NOTREADY name Error
signal on NOVALUE name Error
signal on SYNTAX name Error
parse arg sz
rcode = Initialize()
rcode = ScanArgs( sz )
rcode = ReadFile( Global.FileName )
rcode = ProcessEmail()
rcode = Connect()
rcode = SendEmail()
rcode = Disconnect()
call SysSleep 10

exit 0

/* *************************************************************** */
/* Subroutines                                                     */
/* *************************************************************** */

/* *************************************************************** */
/* SendEmail      send e-mail                                      */
/* *************************************************************** */

SendEmail: procedure expose TRUE FALSE Global. out.

   if Global.Option.Debug then call WriteMessage 'Entering procedure "SendEmail"'
   response = Receive()
   if left( response, 3 ) = '+OK' then
      do
      message = 'user ' || Global.UserID || Global.crlf
      rcode = Send( message )
      response = Receive()
      if left( response, 3 ) = '+OK' then
         do
         if word( response, 2 ) = 'Password' & word( response, 3 ) = 'required' then
            do
            message = 'pass ' || Global.UserPassword || Global.crlf
            rcode = Send( message )
            response = Receive()
            end
         if left( response, 3 ) = '+OK' then
            do
            message = 'xtnd xmit' || Global.crlf
            rcode = Send( message )
            response = Receive()
            if left( response, 3 ) = '+OK' then
               do
               if word( response, 2 ) = 'Start' & word( response, 3 ) = 'sending' then
                  do
                  rcode = Send( out.1 )
                  rcode = Send( out.2 )
                  response = Receive()
                  if left( response, 3 ) = '+OK' then
                     do
                     call WriteMessage substr( response, 5 )
                     end
                  else
                     do
                     call WriteMessage response
                     end
                  end
               end
            else
               do
               call WriteMessage response
               end
            end
         else
            do
            call WriteMessage response
            end
         end
      else
         do
         call WriteMessage response
         end
      end
   else
      do
      call WriteMessage response
      end
   message = 'quit' || Global.crlf
   rcode = Send( message )
   response = Receive()

   return 0

/* *************************************************************** */
/* Send   issue SockSend                                           */
/* *************************************************************** */

Send: procedure expose TRUE FALSE Global.

   if Global.Option.Debug then
      do
      call WriteMessage 'Entering procedure "Send"'
      call WriteMessage 'Sending packet'
      end
   parse arg message
   rcode = SockSend( Global.socket, message )
   if rcode = -1 then call TCPFatal 'SockSend'
   if Global.Option.Debug then
      do
      call WriteMessage 'rc =' rcode
      call WriteMessage 'Sent data =' message
      end

   return rcode

/* *************************************************************** */
/* Receive   issue SockRecv                                        */
/* *************************************************************** */

Receive: procedure expose TRUE FALSE Global.

   if Global.Option.Debug then
      do
      call WriteMessage 'Entering procedure "Receive"'
      call WriteMessage 'Receiving packet'
      end
   rcode = SockRecv( Global.socket, 'response', 512 )
   if rcode = -1 then call TCPFatal 'SockRecv'
   if Global.Option.Debug then
      do
      call WriteMessage 'rc =' rcode
      call WriteMessage 'Received data =' response
      end

   return response

/* *************************************************************** */
/* Process e-mail from Post Road Mailer                            */
/* *************************************************************** */

ProcessEmail: procedure expose TRUE FALSE Global. out. in.

   if Global.Option.Debug then call WriteMessage 'Entering procedure "ProcessEmail"'
   out.1 = ''
   out.2 = ''
   do i = 1 to in.0
      if left( in.i, 1 ) = x2c( '05' ) then
         do
         b2 = substr( in.i, 2, 1 )
         indata = substr( in.i, 3 )
         select
            when b2 = x2c( '01' ) then           /* To: */
               do
               out.1 = 'To: ' || indata || Global.crlf
               end
            when b2 = x2c( '02' ) then           /* CC: */
               do
               /* not implemented */
               end
            when b2 = x2c( '0b' ) then           /* From: */
               do
               out.2 = out.2 || 'From: ' || indata || Global.crlf
               end
            when b2 = x2c( '0f' ) then           /* Subject: */
               do
               out.2 = out.2 || 'Subject: ' || indata || Global.crlf
               end
            when b2 = x2c( '10' ) then           /* Signature block */
               do
               /* ignore */
               end
            when b2 = x2c( '13' ) then           /* Pathname of folder to file the note to */
               do
               /* not implemented */
               end
            when b2 = x2c( '14' ) then           /* Title of folder to file the note to */
               do
               /* not implemented */
               end
            when b2 = x2c( '21' ) then           /* Full pathname of attachment file to be MIMEd */
               do
               /* not implemented */
               end
            when b2 = x2c( '26' ) then           /* Full pathname of attachment file to be UUencoded */
               do
               /* not implemented */
               end
            when b2 = x2c( '31' ) then           /* Reply-to: */
               do
               out.2 = out.2 || 'Reply-To: ' || indata || Global.crlf
               end
            when b2 = x2c( '32' ) then           /* Priority: */
               do
               out.2 = out.2 || 'Priority: ' || indata || Global.crlf
               out.2 = out.2 || 'X-Mailer: ' || Global.CmdName Global.Version  || Global.crlf
               out.2 = out.2 || 'Date: ' || left( date( 'W' ), 3 ) || ', ' || date( 'N' ) time( 'N' ) Global.TZ || Global.crlf
               end
            when b2 = x2c( '34' ) then           /* Tagline */
               do
               /* not implemented */
               end
            when b2 = x2c( '35' ) then           /* Distribution list (To: or CC:) */
               do
               /* not implemented */
               end
            otherwise
               do
               /* ignore because I don't know what it is so it is not implemented */
               end
         end  /* select */
         end
      else
         do                                      /* message text */
         out.2 = out.2 || in.i || Global.crlf
         end
   end /* do i = 1 to in.0 */
   out.2 = out.2 || x2c( '2e' ) || Global.crlf

   return 0

/* *************************************************************** */
/* ReadFile -- Read a File                                         */
/* *************************************************************** */

ReadFile: procedure expose Global. in. TRUE FALSE

   if Global.Option.Debug then call WriteMessage 'Entering procedure "ReadFile"'
   parse arg fn
   rcode = stream( fn, 'C', 'query exists' )
   if rcode = '' then
      do
      call WriteMessage 'File name "' || fn || '" not found'
      call SysSleep 10
      exit 200
      end
   i = 0
   do while lines( fn )
      i = i + 1
      in.i = linein( fn )
   end /* do while lines( fn ) */
   in.0 = i
   rcode = stream( fn, 'C', 'CLOSE' )

   return 0

/* *************************************************************** */
/* Connect(): Connect socket                                       */
/* *************************************************************** */

Connect: procedure expose Global. TRUE FALSE

   if Global.Option.Debug then call WriteMessage 'Entering procedure "Connect"'
   rcode = SockGetHostByName( Global.HostName, 'theHost.!' )
   if \rcode then call TCPFatal 'SockGetHostByName'
   if Global.Option.Debug then
      do
      call WriteMessage 'xxx.!name =' theHost.!name
      call WriteMessage 'xxx.!alias.0 =' theHost.!alias.0
      do i = 1 to theHost.!alias.0
         call WriteMessage 'xxx.!alias.' || i '=' theHost.!alias.i
      end /* do i = 1 to theHost.!alias.0 */
      call WriteMessage 'xxx.!addrtype =' theHost.!addrtype
      call WriteMessage 'xxx.!addr =' theHost.!addr
      call WriteMessage 'xxx.!addr.0 =' theHost.!addr.0
      do i = 1 to theHost.!addr.0
         call WriteMessage 'xxx.!addr.' || i '=' theHost.!addr.i
      end /* do i = 1 to theHost.!addr.0 */
      end
   Global.socket = SockSocket( 'AF_INET', 'SOCK_STREAM', 'IPPROTO_TCP' )
   if Global.socket = -1 then call TCPFatal 'Socket'
   /* SockSetSockOption is broken on object rexx so bypass if testing under OREXX */
   parse version RexxVer .
   if RexxVer == 'OBJREXX' then
      do
      call WriteMessage 'Running OREXX.  Bypassing SockSetSockOpt'
      end
   else
      do
      /* setsockopt(S, SOL_SOCKET, SO_BROADCAST, 1) or die "setsockopt :$!"; */
      rcode = SockSetSockOpt( Global.socket, 'SOL_SOCKET', 'SO_BROADCAST', 1 )
      if rcode = -1 then call TCPFatal 'SockSetSockOpt'
      end
   theHost.!family = 'AF_INET'
   theHost.!port = Global.Port
   rcode = SockConnect( Global.socket, 'theHost.!' )
   if rcode = -1 then call TCPFatal 'SockConnect'

   return 0

/* *************************************************************** */
/* Disconnect(): Disconnect socket                                 */
/* *************************************************************** */

Disconnect: procedure expose Global. TRUE FALSE

   if Global.Option.Debug then call WriteMessage 'Entering procedure "Disconnect"'
   rcode = SockClose( Global.socket )
   if rcode = -1 then call TCPFatal 'SockClose'

   return 0

/* *************************************************************** */
/* ScanArgs( argLine ): scan command line arguments and switches   */
/* *************************************************************** */

ScanArgs: procedure expose Global. TRUE FALSE

   if Global.Option.Debug then call WriteMessage 'Entering procedure "ScanArgs"'
   /* Evaluate arguments - override
       Return Global.Option. and aArgs.
       fixme to do quotes
       Uses work stem Z */
   parse arg szRest
   szRest = strip( szRest )
   szSwCtl = 'fhptu'                     /* Switches that take args */
   szArg = ''
   szSw = ''
   fSwEnd = FALSE                        /* End of switches */
   do while szRest \== '' | szArg \== '' | szSw \== ''
      /* If arg buffer empty, refill */
      if szArg == '' then parse var szRest szArg szRest
      /* If switch buffer empty, refill */
      if szSw == '' then
         do
         if left( szArg, 1 ) == '-' then
            do
            if fSwEnd then
               do
               call Usage 'switch '''szArg''' unexpected'
               end
            else
               do
               if szArg == '--' then
                  do
                  fSwEnd = TRUE
                  end
               else
                  do
                  szSw = substr( szArg, 2 )
                  end
               end
            parse var szRest szArg szRest
            end
         end
      /* If switch in progress */
      if szSw \== '' then
         do
         sz = left( szSw, 1 )                /* Next switch */
         szSw = substr( szSw, 2 )            /* Drop from pending */
         /* Check switch requires argument */
         if pos( sz, szSwCtl ) \= 0 then
            do
            if szSw \== ''  then
               do
               szOpt = szSw
               szSw = ''
               end
            else
               do
               if szArg \== '' & left( szArg, 1 ) \= '-' then
                  do
                  szOpt = szArg
                  parse var szRest szArg szRest
                  end
               else
                  do
                  call Usage 'Switch' sz 'requires argument'
                  end
               end
            end
         select
            when sz == '?' then
               do
               signal UsageHelp
               end
            when sz == 'd' then
               do
               Global.Option.Debug = TRUE
               end
            when sz == 'f' then
               do
               Global.FileName = szOpt
               end
            when sz == 'h' then
               do
               Global.HostName = szOpt
               end
            when sz == 'u' then
               do
               Global.UserID = szOpt
               end
            when sz == 'p' then
               do
               Global.UserPassword = szOpt
               end
            when sz == 't' then
               do
               if datatype( szOpt, 'N' ) then
                  do
                  Global.Port = szOpt
                  end
               else
                  do
                  Global.Port = ''
                  end
               end
            when sz == 'v' then
               do
               call WriteMessage Global.CmdName Global.Version
               call SysSleep 10
               exit
               end
            otherwise
               do
               call Usage 'switch '''sz''' unexpected'
               end
         end /* select */
         end
      else
         do
         if szArg \== '' then
            do
            fSwEnd = TRUE                      /* No more switches */
            /* Got non switch arg */
            szArg = ''
            end
         end
   end /* do while szRest \== '' | szArg \== '' | szSw \== '' */
   if Global.FileName = '' | Global.HostName = '' | Global.UserID = '' | Global.UserPassword = '' | Global.Port = '' then
      do
      message = ''
      if Global.FileName = '' then message = message '-f'
      if Global.HostName = '' then message = message '-h'
      if Global.UserID = '' then message = message '-u'
      if Global.UserPassword = '' then message = message '-p'
      if Global.Port = '' then message = message '-t'
      call WriteMessage 'missing parameters are' message
      call SysSleep 10
      exit
      end

   return 0

/* *************************************************************** */
/* TCPFatal( szReq ): Report TCPFatal Error...                     */
/* *************************************************************** */

TCPFatal:

   parse arg szReq
   call WriteMessage szReq 'failed, errno:' errno ', h_errno:' h_errno
   call WriteMessage 'Ooops'
   if Global.socket \= -1 then
      do
      call WriteMessage 'Closing socket' Global.socket
      call SockClose Global.socket
      end
   call SysSleep 10

   exit 253

/* *************************************************************** */
/* Usage(szMsg): Report Usage Error...                             */
/* *************************************************************** */

Usage:

   parse arg szMsg
   call WriteMessage szMsg
   call WriteMessage 'Usage:' Global.CmdName '?vdu:p:t:f:h:'
   call SysSleep 10

   exit 255

/* *************************************************************** */
/* Usage: Display help                                             */
/* *************************************************************** */

UsageHelp:

   call WriteMessage
   call WriteMessage 'Usage:' Global.CmdName '?vdu:p:t:f:h:'
   call WriteMessage
   call WriteMessage ' -?           This message'
   call WriteMessage ' -h host      Host name'
   call WriteMessage ' -u userid    user ID'
   call WriteMessage ' -p password  user password'
   call WriteMessage ' -t port      Port address, default = 110'
   call WriteMessage ' -f File      File to send'
   call WriteMessage ' -v           Report version'
   call SysSleep 10

   exit 255

/* *************************************************************** */
/* Error: Trap ERROR, FAILURE etc. conditions                      */
/* *************************************************************** */

Error:

   /*=== Returns szCondition or Exits ===*/
   call WriteMessage
   parse source . . szThisCmd
   call WriteMessage condition( 'C' ) 'signaled at' SIGL 'of' szThisCmd
   drop szThisCmd
   call WriteMessage 'Source =' sourceline( SIGL )
   call SysSleep 2
   if condition( 'I' ) == 'CALL' then
      do
      szCondition = condition( 'C' )
      call WriteMessage 'Returning'
      end
   else
      do
      call WriteMessage 'Exiting'
      call SysSleep 10
      exit 255
      end

   return

/* *************************************************************** */
/* WriteMessage: Write a message to STDOUT                         */
/* *************************************************************** */

WriteMessage: procedure expose TRUE FALSE Global.

   parse arg szMsg
   say Global.CmdName':' szMsg

   return

/* *************************************************************** */
/* Halt: Trap HALT condition                                       */
/* *************************************************************** */

Halt:

   /*=== Returns szCondition or Exits ===*/
   call WriteMessage
   parse source . . szThisCmd
   call WriteMessage condition( 'C' ) 'signaled at' SIGL 'of' szThisCmd
   drop szThisCmd
   call WriteMessage 'Source = ' sourceline( SIGL )
   call SysSleep 2
   if condition( 'I' ) == 'CALL' then
      do
      szCondition = condition( 'C' )
      call WriteMessage 'Returning'
      end
   else
      do
      call WriteMessage 'Exiting'
      call SysSleep 10
      exit
      end

   return

/* *************************************************************** */
/* ToLower(sz): Convert to lower case                              */
/* *************************************************************** */

ToLower: procedure expose TRUE FALSE Global.

   if Global.Option.Debug then call WriteMessage 'Entering procedure "ToLower"'
   parse arg sz
   return translate( sz, xrange( 'a', 'z' ), xrange( 'A', 'Z' ) )

/* *************************************************************** */
/* Initialize: Intialize globals                                   */
/* *************************************************************** */

Initialize:

   TRUE = 1 = 1
   FALSE = \TRUE
   /* Add all Rexx functions */
   rcode = RxFuncAdd( 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs' )
   rcode = SysLoadFuncs()
   /* Load socket functions */
   rcode = RxFuncAdd( 'SockLoadFuncs', 'rxSock', 'SockLoadFuncs' )
   rcode = SockLoadFuncs()
   parse source . . Global.CmdName
   Global.CmdName = filespec( 'N', Global.CmdName )
   c = pos( '.', Global.CmdName )
   if c > 1 then Global.CmdName = substr( Global.CmdName, 1, c - 1 )
   Global.Version = 'v0.1'
   Global.FileName = ''
   Global.HostName = ''
   Global.UserID = ''
   Global.UserPassword = ''
   Global.Port = 110                         /* service 110 = POP */
   Global.Option.Debug = FALSE
   Global.crlf = x2c( '0d0a' ) 
   Global.TZ = substr( value( 'TZ', , 'OS2ENVIRONMENT' ), 1, 3 )

   return 0

